@@ -1,34 +1,38 @@
-class DummyFilterChoices:
+from django import forms
 
-    def __init__(self, field_name):
-        self.field_name = field_name
-
-    def get(self, x, *args, **kwargs):
-        return {self.field_name: x} if x else {}
+__all__ = (
+    'FilterSpec',
+    'RuntimeAwareFilterSpecMixin',
+)
 
 
 class FilterSpec(object):
-
     creation_counter = 0
+    field_cls = forms.CharField
 
-    filter_choices = {}
-
-    def __init__(self, field_name, filter_field=None):
+    def __init__(self, field_name, filter_field=None, **field_kwargs):
         self.field_name = field_name
 
-        if not self.filter_choices:
-            self.filter_choices = DummyFilterChoices(field_name=field_name)
-
         if filter_field is not None:
             self.filter_field = filter_field
+        else:
+            base_kwargs = self.get_field_kwargs()
+            base_kwargs.update(field_kwargs)
+            self.filter_field = (self.get_field_cls(), base_kwargs)
 
         self.creation_counter = FilterSpec.creation_counter + 1
         FilterSpec.creation_counter = self.creation_counter
 
         super(FilterSpec, self).__init__()
 
+    def get_field_cls(self):
+        return self.field_cls
+
+    def get_field_kwargs(self):
+        return {'required': False}
+
     def to_lookup(self, cleaned_value):
-        return self.filter_choices.get(cleaned_value, {})
+        return {self.field_name: cleaned_value} if cleaned_value else {}
 
 
 class RuntimeAwareFilterSpecMixin(object):
